/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>
#include <stdio.h>

// The core of a MIDI plugin
void Module::OnMidiData( unsigned long p_clock, unsigned long midi_msg, short pin_id)
{
	// system messages are for timing info etc
	// they don't have a channel, just pass them right through
	bool is_system_msg = ( midi_msg & SYSTEM_MSG ) == SYSTEM_MSG;


	

	if(( !is_system_msg )&&(getPin(PN_ON)->getValue() >= 0.1 ))
	{
		// The first byte of a midi message contains the status (note on/off etc) and the channel
		//int chan = midi_msg & 0x0f;
		stat = midi_msg & 0xf0;

		// the next two bytes depend on the type of message
		b2 = ( midi_msg / 0x100 ) & 0xff;//note
		b3 = ( midi_msg / 0x10000 ) & 0xff;//vel

       if(trigger == true)
			{
		            float *tryit = offset + output1_buffer;
					for(int i = 0;i<96;i++)
					{
						 
						*tryit = 0.0f;
                         tryit++;
					}
					trigger = false;
			}




		if(stat ==  NOTE_ON)
			{
				if(OU) vdif = vel -  b3; else vdif = b3  - vel;

				if (( b2 == note ) && ( vdif >= 0)) 
				{
					float *tryit = offset + output1_buffer;
					
					for(int i = 0;i<96;i++)
					{
						 *tryit = 0.8f;
						 tryit++;
					}
					trigger = true;
				}
			}
		
		
	}

}



void Module::sub_process(long buffer_offset, long sampleFrames )
{
	
	offset = buffer_offset;
/*
	float *in     = buffer_offset + input1_buffer;
    float *out    = buffer_offset + output1_buffer;

	while(--sampleFrames >= 0)
	{	
		
		if (*in > 0.2)
		{
			
			*out = (float) trigger; 
			*(out+1) = *out;
			if (( b2 == note ) && ( b3 <= vel)) 
			{
			  *(out+2) = 1.0f;
			  b2 = 0;
			  b3 = 127;
			}
		}
		
		in++;
		out++;
	}
	trigger = false;
	*/
	sampleFrames = 0;
}


Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	trigger = false;
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);
}

Module::~Module()
{
	// nothing to see here, move along.
}
// describe the module
bool Module::getModuleProperties (SEModuleProperties* properties)
{ 
   LL dish;
   BB bish;
   sprintf_s(dish.thisSEM   ,100, "%s\n",  "MIDIvel_TRIG");
   int j;

   j  = sprintf_s(  bish.buffer    ,1000, "%s\n", " ************************************" );
   j  = sprintf_s(  bish.buffer + j,1000, "%s\n", " * 2010 March    in Tbingen GERMANY " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *            FAKE MUSIC             " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *         by ralph.a.cronin         " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s"  ,   dish.thisSEM );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " * note under velocity triggers      " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *          enjoy SYNTHEDIT          " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *        by Jeff Mc Clintock        " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );

   // MAXIS :
   // j += sprintf( buffer + j, "%s\n", " **************************************************************************************" );//120
   // j += sprintf( buffer + j, "%s\n", "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" ); // = 54


	properties->name = dish.thisSEM;	// this is the name the end-user will see

	// return a uni+																																																			que string.
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug. No two plugs may have the same id.
	properties->id = dish.thisSEM;

	// Info, may include Author, Web page whatever
	properties->about =  bish.buffer;//"by ralph a. cronin and Jeff M";
	return true;
  
}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )
	{
	case 0:
		properties->name				= "MIDI In";
		properties->direction			= DR_IN;
		properties->datatype			= DT_MIDI2;
		break;
    case 1:
		properties->name				= "ON";
        properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 2:
		properties->name				= "NOTE trigs";
		properties->variable_address	= &note;
		properties->datatype_extra		= "range 0,128";
		properties->default_value		= "0";
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		break;
    case 3:
		properties->name				= "VEL";
		properties->variable_address	= &vel;
		properties->datatype_extra		= "range 0,128";
		properties->default_value		= "40";
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		break;
	case 4:
	    properties->name				= "TRIGG OUT";
        properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
    case 5:
		properties->name				= "1: note < vel";
		properties->variable_address	= &OU;
		properties->datatype_extra		= "range 0,1";
		properties->default_value		= "0";
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}


void Module::OnPlugStateChange(SEPin *pin)
{
	//if(getPin(PN_ON)->getValue() >= 0.1 ) 
		getPin(PN_MIDITRIG)->TransmitStatusChange( SampleClock(), ST_RUN);// statt out_stat ralph
	//else
      //  getPin(PN_MIDITRIG)->TransmitStatusChange( SampleClock(), ST_STATIC);

}